function [GIRF1bands, GIRF2bands] = bootstrapIVAR(IVAR, state, nboot, alpha, method, fast)

% unpacking
data = IVAR.data;
c_case = IVAR.c_case;
t = IVAR.t;
ub= IVAR.u;
n = IVAR.n;
u = IVAR.u;
A = IVAR.A;
p = IVAR.p;
%IRFb = NaN*ones(size(VAR.IRF,1), size(VAR.IRF,2), nboot);
shockpos = state.shockpos;
statepos = state.statepos;
h = IVAR.GIRF1opt.h;
impulse = IVAR.GIRF1opt.impulse;

% define upper and lower thresholds
if isempty(alpha)
    alpha = 90;
end
lo = (100-alpha)/2;
up = 100 - lo;

GIRF1b = NaN*ones(h,n,nboot);
GIRF2b = NaN*ones(h,n,nboot);

nexplos=0;
nattempts = 1;
disp('Progress \n')
bb = 1;
while bb <= nboot
    
    % 1. generate pseudo-disturbance 'ub'
    if strcmp(method, 'residual')
        % residual bootstrapping assumes E(y|Xb) = X*b and u are iid (i.e.
        % homoskedasticity => Draw from the empirical distribution of u
        
        segment = (1:(t-p))/(t-p);
        ub = zeros(t,n);
        for i=1:size(ub,1)
            draw = rand(1,1);  % uniform distribution
            ub(i,:) = u(min(find(segment>=draw)),:);
        end
        
    elseif strcmp(method, 'wild')
        % allows for heteroskedasticity
        
        fu = 1-2*(rand(t-p,1)>0.5);
        ub = zeros(t-p, n);
        for i=1:n
            ub(:,i) = u(:,i).*fu;  % flip sign of randomly selected 50%
        end
        ub = [zeros(p,n); ub];
        
    end
    
    % 2. generate pseudo-sample based on drawn u's
    % Important: The 'exogenous' interactions of endogenous variables have
    % to be constructed manually, since they directly depend on some of the
    % simulated/bootstrapped data.
    % 2a. Initial values for the artificial data: real data + plus
    % artificial residuals
    LAG = [];
    for i = 1:p
        datab(i,:) = data(i,:) + ub(i,:);
        LAG = [datab(i,:) LAG];
        if c_case == 0
            LAGplus = LAG;
        elseif c_case == 1
            LAGplus = [1 LAG];
        elseif c_case == 2
            LAGplus = [1 i LAG];
        end
    end
    
    % 2b. construct interaction terms of initial observations
    if statepos~=0
        if p==1
            LAGplus = [LAGplus LAG(1,shockpos)*LAG(1,statepos)];
        elseif p==2
            LAGplus = [LAGplus LAG(1,shockpos)*LAG(1,statepos) LAG(1,shockpos+n)*LAG(1,statepos+n)];
        elseif p==3
            LAGplus = [LAGplus LAG(1,shockpos)*LAG(1,statepos) LAG(1,shockpos+n)*LAG(1,statepos+n) LAG(1,shockpos+2*n)*LAG(1,statepos+2*n)];
        elseif p==4
            LAGplus = [LAGplus LAG(1,shockpos)*LAG(1,statepos) LAG(1,shockpos+n)*LAG(1,statepos+n) LAG(1,shockpos+2*n)*LAG(1,statepos+2*n) LAG(1,shockpos+3*n)*LAG(1,statepos+3*n)];
        elseif p==5
            LAGplus = [LAGplus LAG(1,shockpos)*LAG(1,statepos) LAG(1,shockpos+n)*LAG(1,statepos+n) LAG(1,shockpos+2*n)*LAG(1,statepos+2*n) LAG(1,shockpos+3*n)*LAG(1,statepos+3*n) LAG(1,shockpos+4*n)*LAG(1,statepos+4*n)];
        elseif p==6
            LAGplus = [LAGplus LAG(1,shockpos)*LAG(1,statepos) LAG(1,shockpos+n)*LAG(1,statepos+n) LAG(1,shockpos+2*n)*LAG(1,statepos+2*n) LAG(1,shockpos+3*n)*LAG(1,statepos+3*n) LAG(1,shockpos+4*n)*LAG(1,statepos+4*n) LAG(1,shockpos+5*n)*LAG(1,statepos+5*n)];
        elseif p==7
            LAGplus = [LAGplus LAG(1,shockpos)*LAG(1,statepos) LAG(1,shockpos+n)*LAG(1,statepos+n) LAG(1,shockpos+2*n)*LAG(1,statepos+2*n) LAG(1,shockpos+3*n)*LAG(1,statepos+3*n) LAG(1,shockpos+4*n)*LAG(1,statepos+4*n) LAG(1,shockpos+5*n)*LAG(1,statepos+5*n) LAG(1,shockpos+6*n)*LAG(1,statepos+6*n)];
        elseif p==8
            LAGplus = [LAGplus LAG(1,shockpos)*LAG(1,statepos) LAG(1,shockpos+n)*LAG(1,statepos+n) LAG(1,shockpos+2*n)*LAG(1,statepos+2*n) LAG(1,shockpos+3*n)*LAG(1,statepos+3*n) LAG(1,shockpos+4*n)*LAG(1,statepos+4*n) LAG(1,shockpos+5*n)*LAG(1,statepos+5*n) LAG(1,shockpos+6*n)*LAG(1,statepos+6*n) LAG(1,shockpos+7*n)*LAG(1,statepos+7*n)];
        elseif p==9
            LAGplus = [LAGplus LAG(1,shockpos)*LAG(1,statepos) LAG(1,shockpos+n)*LAG(1,statepos+n) LAG(1,shockpos+2*n)*LAG(1,statepos+2*n) LAG(1,shockpos+3*n)*LAG(1,statepos+3*n) LAG(1,shockpos+4*n)*LAG(1,statepos+4*n) LAG(1,shockpos+5*n)*LAG(1,statepos+5*n) LAG(1,shockpos+6*n)*LAG(1,statepos+6*n) LAG(1,shockpos+7*n)*LAG(1,statepos+7*n) LAG(1,shockpos+8*n)*LAG(1,statepos+8*n)];
        else
            disp('Bootstrap only available up to p = 9. Please write code forward if more are needed.')
        end
    end
    
    % 3. generate artificial data from observation p+1 forward
    for i = (p+1):t
        
        for nn = 1:n % forward-simulate endogenous variables
            datab(i,nn) = LAGplus * A(1:end,nn) + ub(i,nn);
        end
        LAG = [datab(i,:) LAG(1,1:(p-1)*n)]; % update the LAG matrix
        if c_case == 0
            LAGplus = LAG;
        elseif c_case==1
            LAGplus = [1 LAG];
        elseif c_case==2
            LAGplus = [1 i LAG];
        end
        
        if statepos~=0 % construct the interaction variables
            if p==1
                LAGplus = [LAGplus LAG(1,shockpos)*LAG(1,statepos)];
            elseif p==2
                LAGplus = [LAGplus LAG(1,shockpos)*LAG(1,statepos) LAG(1,shockpos+n)*LAG(1,statepos+n)];
            elseif p==3
                LAGplus = [LAGplus LAG(1,shockpos)*LAG(1,statepos) LAG(1,shockpos+n)*LAG(1,statepos+n) LAG(1,shockpos+2*n)*LAG(1,statepos+2*n)];
            elseif p==4
                LAGplus = [LAGplus LAG(1,shockpos)*LAG(1,statepos) LAG(1,shockpos+n)*LAG(1,statepos+n) LAG(1,shockpos+2*n)*LAG(1,statepos+2*n) LAG(1,shockpos+3*n)*LAG(1,statepos+3*n)];
            elseif p==5
                LAGplus = [LAGplus LAG(1,shockpos)*LAG(1,statepos) LAG(1,shockpos+n)*LAG(1,statepos+n) LAG(1,shockpos+2*n)*LAG(1,statepos+2*n) LAG(1,shockpos+3*n)*LAG(1,statepos+3*n) LAG(1,shockpos+4*n)*LAG(1,statepos+4*n)];
            elseif p==6
                LAGplus = [LAGplus LAG(1,shockpos)*LAG(1,statepos) LAG(1,shockpos+n)*LAG(1,statepos+n) LAG(1,shockpos+2*n)*LAG(1,statepos+2*n) LAG(1,shockpos+3*n)*LAG(1,statepos+3*n) LAG(1,shockpos+4*n)*LAG(1,statepos+4*n) LAG(1,shockpos+5*n)*LAG(1,statepos+5*n)];
            elseif p==7
                LAGplus = [LAGplus LAG(1,shockpos)*LAG(1,statepos) LAG(1,shockpos+n)*LAG(1,statepos+n) LAG(1,shockpos+2*n)*LAG(1,statepos+2*n) LAG(1,shockpos+3*n)*LAG(1,statepos+3*n) LAG(1,shockpos+4*n)*LAG(1,statepos+4*n) LAG(1,shockpos+5*n)*LAG(1,statepos+5*n) LAG(1,shockpos+6*n)*LAG(1,statepos+6*n)];
            elseif p==8
                LAGplus = [LAGplus LAG(1,shockpos)*LAG(1,statepos) LAG(1,shockpos+n)*LAG(1,statepos+n) LAG(1,shockpos+2*n)*LAG(1,statepos+2*n) LAG(1,shockpos+3*n)*LAG(1,statepos+3*n) LAG(1,shockpos+4*n)*LAG(1,statepos+4*n) LAG(1,shockpos+5*n)*LAG(1,statepos+5*n) LAG(1,shockpos+6*n)*LAG(1,statepos+6*n) LAG(1,shockpos+7*n)*LAG(1,statepos+7*n)];
            elseif p==9
                LAGplus = [LAGplus LAG(1,shockpos)*LAG(1,statepos) LAG(1,shockpos+n)*LAG(1,statepos+n) LAG(1,shockpos+2*n)*LAG(1,statepos+2*n) LAG(1,shockpos+3*n)*LAG(1,statepos+3*n) LAG(1,shockpos+4*n)*LAG(1,statepos+4*n) LAG(1,shockpos+5*n)*LAG(1,statepos+5*n) LAG(1,shockpos+6*n)*LAG(1,statepos+6*n) LAG(1,shockpos+7*n)*LAG(1,statepos+7*n) LAG(1,shockpos+8*n)*LAG(1,statepos+8*n)];
            end
        end
    end
    
    acceptdraw = 1;
    if isnan(datab(t,shockpos))
        acceptdraw = 0;
    else
        % 4. Estimate VAR, repeat if leads to errors (can happen due to
        % explosiveness of artificial data)
        if statepos==0
            exdatab = [];
        else % if IVAR, fill in the exogenous 
            exdatab = datab(:,shockpos).*datab(:,statepos);
        end
        warning('error', 'MATLAB:nearlySingularMatrix');  
        warning('off','all')        
        try
            IVARb = estimateVAR(datab, p, c_case, exdatab);
        catch % if error occurs
            acceptdraw = 0;
        end

        % 5. Generalized impulse response functions
        if acceptdraw == 1
            if statepos == 0
                [GIRF1b(:,:,bb), GIRF1bopt] = simGIRF(IVARb, state, h, ones(size(state.s,1),1), impulse, fast, 0);
            else % if IVAR, split histories in 2 and save both
                sb = logical(datab(:,statepos) <= state.absval);
                [GIRF1b(:,:,bb), GIRF1bopt] = simGIRF(IVARb, state, h, ~sb(p+1:end), impulse, fast, 0);
                [GIRF2b(:,:,bb), ~] = simGIRF(IVARb, state, h, sb(p+1:end), impulse, fast, 0);
            end
        end
        if GIRF1bopt.nexplos == GIRF1bopt.nhist
            acceptdraw = 0;
        end
        warning ('on','all'); % restore warning state
        
    end
    
    % 6. Decide whether or not to keep
    if acceptdraw == 1 % save and print progress bar
        fprintf(1,'\b\b\b\b%3.0f%%',100*(bb/nboot));
        bb = bb+1;
    else
        nexplos = nexplos + 1;
    end
    nattempts = nattempts + 1;
    
end
fprintf('\n');

% number of interrupted bootstrap iterations
if (nexplos/nattempts) > 0.5
    disp(['Warning: ', num2str(round(100*(nexplos/nattempts))), '% of bootstrap iterations were discarded. Possible fixes:']);
    disp(' - Start the sample at another date.')
    disp(' - Make sure number of disregarded histories is close to zero in main GIRF function.')
    disp(' - Choose a different bootstrap method.')
end
% retrieve intervals
GIRF1bands=prctile(GIRF1b, [lo up],3);
GIRF2bands=prctile(GIRF2b, [lo up],3);
