function [AIC, BIC, HQC] = InfCriteria(data, p, c_case, exdata)
%%


% Prep
t = size(data, 1);
n = size(data, 2);  

%% Y matrix
Y = data;

%% X matrix: Generate lags
X = [];
for pp = 1:p
    X = [X, [NaN*ones(pp, n); data(1:end-pp,:)] ];
end
% lags introduce some NA's => truncate
X = X(p+1:end,:); 
Y = Y(p+1:end,:);
% add constant and/or trend to X matrix
if c_case == 0
    c = [];
elseif c_case == 1
    c = ones(size(X,1),1); 
elseif c_case == 2
    c = [ones(size(X,1),1), [1:1:size(X)]' ];
else
    print('c_case variable needs to be set to 0, 1, or 2.')
end
% add exogenous variables
if ~isempty(exdata)
    Xex = [];
    for pp = 1:p
        Xex = [Xex, [NaN*ones(pp, size(exdata,2)); exdata(1:end-pp,:)]  ];
    end
    Xex = Xex(p+1:end,:);
else
    Xex = [];
end
X = [c, X, Xex];

%% Estimation
A = X\Y;  % a_ij is reaction of column j to row i
u = Y - X*A;  % reduced-form residuals
Omega = cov(u);  % variance-covariance matrix
S = chol(Omega)';  % lower-triangular matrix (Cholesky decomposition)

%% Information criteria

log_lik = nan(t-p,1);
for tt=1:t-p
    log_lik(tt) = -n*0.5*log(2*pi)-0.5*log(det(Omega))...
        -0.5*u(tt,:)/Omega*u(tt,:)';
end
log_lik = sum(log_lik);

numPar = n+n^2*p;

AIC = log(det(Omega))+2/t*numPar;
BIC = log(det(Omega))+2*log(log(t))/t*numPar;
HQC = log(det(Omega))+log(t)/t*numPar;


