function CFT = cft_VAR(VAR, Boot, alpha, cf)

% Unpacking
t = size(VAR.u,1);
n = VAR.n;
A = VAR.A;
C = VAR.C;
c_case = VAR.c_case;
Xex = VAR.Xex;
p = VAR.p;
h = size(VAR.IRF,1);
nboot = size(Boot.Ab,3);
T = cf.T;
hz = cf.hz;
method = cf.method;
shockpos = cf.shockpos;
if strcmp(method, 'shockseries')
    shockseries = zeros(n, hz);
    shockseries(shockpos,1:size(cf.shockseries,2)) = cf.shockseries;
elseif strcmp(method, 'targetseries')
    targetseries = cf.targetseries;
    shockseries = zeros(n, hz);
end

%% Creating baseline scenario: simulate actual data with actual u's
Y = VAR.data(p+1:end,:);
Y = Y(1:T,:);
X = VAR.X(1:T,:);
u =  [VAR.u; zeros(hz, n) ]; 

%Build xlag matrix    
if c_case == 0
    xlag = [Y(end,:), X(end,1:end-n)];
elseif c_case == 1
    xlag = [ones(1,1),Y(end,:),X(end,1:end-n)];
elseif c_case == 2
    xlag = [ones(1,1), T+1, Y(end,:), X(end,1:end-n)]; 
end

for tt = (T+1):1:(T+max(hz))
    % deterministic part
    Y_det = xlag*A;
    % stochastic part (shocks before T)
    Y_stoch = u(tt,:);
    Y(end+1,:) = Y_det + Y_stoch;
    if c_case == 0
        xlag = [Y(end,:), xlag(1:end-n)];
    elseif c_case == 1
        xlag = [ones(1,1), Y(end,:), xlag(2:end-n)];
    elseif c_case == 2
        xlag = [ones(1,1), tt+1, Y(end,:), xlag(3:end-n)];
    end
end
CFT.Pre = Y(1:T,:);
CFT.BL = Y((T+1):(T+hz),:);


%% Creating alternative scenario
         
Y_alt = VAR.data(p+1:end,:);
Y_alt = Y_alt(1:T,:);
X_alt = VAR.X(1:T,:);
u_alt = [VAR.u; zeros(hz, n) ];  % shocks after T are zero
if strcmp(method, 'targetseries') && size(targetseries,2) < hz
    targetseries = [targetseries, Y((T+size(targetseries,2)+1):(T+hz),shockpos)'];
end

%Build xlag matrix    
if c_case == 0
    xlag = [Y_alt(end,:), X_alt(end,1:end-n)];
elseif c_case == 1
    xlag = [ones(1,1), Y_alt(end,:), X_alt(end,1:end-n)];
elseif c_case == 2 
    xlag = [ones(1,1), T+1, Y_alt(end,:), X_alt(end,1:end-n)]; 
end

for tt = (T+1):1:(T+max(hz))
    % deterministic part
    Y_det = xlag*A;
    % stochastic part
    if strcmp(method, 'shockseries')
        c_shock = VAR.S(shockpos, shockpos) * shockseries(:,tt-T)' * inv(VAR.S); %Creating counterfactual (orthogonal) shock
    elseif strcmp(method, 'targetseries')
        % required u to achieve target 
        shockseries(shockpos,tt-T) = targetseries(tt-T) - Y_det(shockpos) - u_alt(tt,shockpos);
        c_shock = VAR.S(shockpos, shockpos) * shockseries(:,tt-T)' * inv(VAR.S);
    end
    Y_alt(end+1,:) = Y_det + u_alt(tt, :) + c_shock;
    if c_case == 0
        xlag = [Y_alt(end,:), xlag(1:end-n)];
    elseif c_case == 1
        xlag = [ones(1,1), Y_alt(end,:), xlag(2:end-n)];
    elseif c_case == 2
        xlag = [ones(1,1), tt+1, Y_alt(end,:), xlag(3:end-n)];
    end
end

%Storing alternative scenario
CFT.Scenario = Y_alt((T+1):(T+hz),:);
CFT.Diff = CFT.Scenario - CFT.BL;



%% Running bootstrap for the alternative scenario
% define upper and lower thresholds
if isempty(alpha)
    alpha = 90;
end
lo = (100-alpha)/2;
up = 100 - lo;

for bb = 1:nboot
    
    Yb = VAR.data(p+1:end,:);
    Yb = Yb(1:T,:);
    Xb = VAR.X(1:T,:);
    ub = [ VAR.u; zeros(hz,n) ];  % shocks after T are zero
    if strcmp(method, 'targetseries')
        shockseries = zeros(n, hz);
    end

    if c_case == 0
        xlag = [Yb(end,:), Xb(end,1:end-n)];
    elseif c_case == 1
        xlag = [ones(1,1), Yb(end,:), Xb(end,1:end-n)];
    elseif c_case == 2
        xlag = [ones(1,1), T+1, Yb(end,:), Xb(end,1:end-n)]; 
    end

    for tt = (T+1):1:(T+max(hz))
        % deterministic part
        Y_det = xlag*Boot.Ab(:,:,bb); 
        % stochastic part
        if strcmp(method, 'shockseries')
            c_shock = VAR.S(shockpos, shockpos) * shockseries(:,tt-T)' * inv(VAR.S); %Creating counterfactual shock
        elseif strcmp(method, 'targetseries')
            % required u to achieve target 
            shockseries(shockpos,tt-T) = targetseries(tt-T) - Y_det(shockpos) - ub(tt,shockpos);
            c_shock = VAR.S(shockpos, shockpos) * shockseries(:,tt-T)' * inv(VAR.S);
     end
     Yb(end+1,:) = Y_det + ub(tt, :) + c_shock;
     if c_case == 0
         xlag = [Yb(end,:), xlag(1:end-n)];
     elseif c_case == 1
         xlag = [ones(1,1), Yb(end,:), xlag(2:end-n)];
     elseif c_case == 2
         xlag = [ones(1,1), tt+1, Yb(end,:), xlag(3:end-n)];
     end
    end
    
    CFT.ScenarioB(:,:,bb) = Yb((T+1):(T+hz),:);
    
end

CFT.Scenariobands=prctile(CFT.ScenarioB, [lo up], 3);
CFT.Diffbands = CFT.Scenariobands - CFT.BL;

end
