
addpath('./_tbx/lp_tbx') 
addpath('./_tbx/var_tbx') 


%% Settings
folder = 'c02_lplin/chol';
p = 12;
h = 48;
c_case = 1;
vars = {'GDPgap','Unemp','CoreCPIGr12','FFR'};
exvars = {};
n = length(vars);
ident = 'chol';
shockpos = 4; % position of the shock in the Cholesky ordering
shocksize = 0; 
state.nonlinear = 'no';
alpha = 90; %confidence level


%% Load data, generate lags, subset relevant subsample, etc.
load('data/data_m_ready.mat')
subset_data;

%% Estimate dynamic multipliers
LP = estimateLP(data, h, p, c_case, [], alpha);

%% Identification: Estimate VAR to get S
VAR = estimateVAR(data, p, c_case, []);
LP.ident = ident;
LP.shock = zeros(n,1);
LP.shock(shockpos) = 1;
if shocksize ~= 0  % absolute values, e.g. 25bp = 0.25
    LP.shock = LP.shock ./ VAR.S(shockpos, shockpos) .* shocksize;
end
% structural impulse responses
LP.IRF = zeros(h,n);
LP.IRFbands = zeros(h,n,1);
for hh=1:h
    LP.IRF(hh,:) = (LP.Gamma(:,:,hh)*VAR.S*LP.shock)';
    LP.IRFbands(hh,:,1) = (LP.Gammalo(:,:,hh)*VAR.S*LP.shock)';
    LP.IRFbands(hh,:,2) = (LP.Gammaup(:,:,hh)*VAR.S*LP.shock)';
end


%% Plot impulse responses
plotirf1(LP.IRF, LP.IRFbands, printvars, strcat(folder,'lin'))
save(strcat(folder, 'out.mat'), 'LP')