
addpath('./_tbx/var_tbx') 
addpath('./_tbx/stvar_tbx') 

%% Settings
folder = 'c06_stvar/replicateAG_';
p = 3; % quarterly data w/ 3 lags
h = 20;
c_case = 1;
vars = {'G','T','Y'}; % only for labelling, data is loaded directly
printvars = {'Government spending','Taxes','Real output'};
n = length(vars);
ident = 'chol';
shockpos = 1;
shocksize = 1;
state.nonlinear = 'yes';
state.logistic = 'yes';
state.interacted = 'no';
state.statevar = 'MA7';
state.gamma = 3;
state.cq = 30;
state.Fs = 1; % not used
nboot = 10000;
alpha = 90;


%% Load data from AG file
load('data_AG_original.mat')
data = s_data.Y0;
exdata = s_exo.FX(:,1);
state.s = s_data.Z0;
state.Fs = exp(-state.gamma*state.s)./(1+exp(-state.gamma*state.s));   

%% Estimate matrices A, Omega, S, dynamic multipliers incl. Cholesky identification
STVAR = estimateSTVAR(data, state, shockpos, shocksize, p, c_case, exdata, h, nboot, alpha);

%% Plot impulse responses and save VAR structure
plotirf2(STVAR.IRF1, STVAR.IRF1bands, STVAR.IRF2, STVAR.IRF2bands, printvars, strcat(folder,'nonlin'), {'Expansion','Recession'}, 'SouthWest')
save(strcat(folder, 'out.mat'), 'STVAR')