
addpath('./_tbx/var_tbx') 
addpath('./_tbx/ivar_tbx') 


%% Settings
folder = 'c07_ivar/';
p = 4;
h = 25;
c_case = 1;
vars = {'GDPgap','Unemp','CoreCPIGr4','FFRshadow'};
exvars = {}; % has to be empty in IVAR - will be constructed in subset_data.m
n = length(vars);
ident = 'chol';
shockpos = 4; % position of the shock in the Cholesky ordering
shocksize = 0; % 0 = one standard deviation, all else: absolute values
state.nonlinear = 'yes';
state.logistic = 'no';
state.interacted = 'yes';
state.shockvar = {'FFRshadow'}; % both 'shockvar' and 'shockpos' need to be part of 'vars'
state.statevar = {'GDPgap'}; 
state.cq = 12;
nboot = 1000;
alpha = 90; %confidence level


%% Load data, generate lags, subset relevant subsample, etc.
load('data/data_q_ready.mat') % quarterly!
subset_data;

%% Estimate a linear VAR with interaction term in exdata
IVAR = estimateVAR(data, p, c_case, exdata);

%% Split sample into histories of two groups and simulate generalized impulse responses
[IVAR.GIRF1, IVAR.GIRF1opt] = simGIRF(IVAR, state, h, ~state.s(p+1:end), shocksize, 0, 1); % do h + a number to make sure GIRFs are stable
[IVAR.GIRF2, IVAR.GIRF2opt] = simGIRF(IVAR, state, h, state.s(p+1:end), shocksize, 0, 1);

%% Bootstrap
[IVAR.GIRF1bands, IVAR.GIRF2bands] = bootstrapIVAR(IVAR, state, nboot, alpha, 'residual', 1);

%% Figure and export
plotirf2(IVAR.GIRF1(1:h,:), IVAR.GIRF1bands(1:h,:,:), IVAR.GIRF2(1:h,:), IVAR.GIRF2bands(1:h,:,:), printvars, strcat(folder,'nonlin'), {'Boom','Recession'}, [])
save(strcat(folder, 'out.mat'), 'IVAR')
