% 3-EQUATION NK MODEL

// 1. Declare model variables
var y, pi, r, zy, zi, zpi, qe, theta, // model variables
    y_obs, pi_obs, r_obs, qe_obs; // observation variables
varexo epsy epsi epspi epsqe epstheta; // structural shocks

// 2. Declare parameters
parameters betta sigma psi phiy phipi omega alphha epsilon Omega gamma lambda kappa rhoy rhoi rhopi rhoqe rhotheta zz b1 b2;

betta=0.99;  % Discount factor - Dynare cannot handle name 'beta'
sigma = 1;    % CRRA parameters, 1=log utility
psi = 1;      % Labor supply parameter (Frisch elasticity)
phiy  =0.5/4;   % Taylor rule: feedback on output gap
phipi =1.5;   % Taylor rule: feedback on expected inflation
omega =2/3;  % Calvo parameter of price stickiness
gamma = ((1-omega)*(1-betta*omega)/omega);
rhoy = 0.8;  % persistence of agg. demand shock
rhopi = 0.8;   %persistence of cost-push shock
rhoi = 0.5;     %persistence of monetary policy shock
rhotheta = 0.8; % persistence of credit shock
rhoqe = 0.8; % persistence of qe shock
b1 = 0.7;
b2 = 1-b1;
zz = 0.33;

// 3. Declare model equations
model(linear);
y = y(+1)-((1-zz)/sigma)*(r-pi(+1))-zz*(b1*(theta(+1) - theta) + b2*(qe(+1) - qe))+zy;
pi = betta*pi(+1) + gamma*psi*((1-zz)+sigma)/(1-zz)*y - (zz/(1-zz))*gamma*sigma*(b1*theta + b2*qe) + zpi;
r = phipi*pi+phiy*y+zi;
zy = rhoy*zy(-1)+epsy;  // aggregate demand shock
zpi = rhopi*zpi(-1)+epspi;  // cost-push shock
zi = rhoi*zi(-1)+epsi;  // monetary policy shock
theta = rhotheta*theta(-1)+epstheta; // credit shock
qe = rhoqe*qe(-1)+epsqe; // QE shock
// Observation equations: match the model variables to the observed series
y_obs=y;
pi_obs=pi;
r_obs=r;
qe_obs=qe;
end;

steady;

// 4. Declare observed variables
varobs y_obs, pi_obs, r_obs, qe_obs;

// 5. Declare priors for estimated parameters
estimated_params; 
stderr epsy, INV_GAMMA_PDF,1,inf;
stderr epspi, INV_GAMMA_PDF,1,inf;
stderr epsi, INV_GAMMA_PDF,1,inf;
stderr epsqe, INV_GAMMA_PDF,1,inf;
end;

// 6. Estimation
estimation(datafile=data_for_dynare,mode_compute=2,prefilter=1,mh_replic=20000,mh_jscale=0.9, mh_drop=0.2); 

// 7. Evaluate results
shock_decomposition y_obs pi_obs r_obs;
stoch_simul(irf=16) y pi r qe;