
addpath('./_tbx/dsge_tbx/Dynare 4.5.4/matlab') 


%% Simulation (calling dynare)
folder = 'c08_dsge_swzqe/';
vars = {'GDPgap','dCoreCPI','FFRdev','QE'};
printvars = {'Output','Inflation','Interest rate','QE'};

%% Load data and create 'data_for_dynare.mat'
load('data/data_q_ready.mat'); % load data library
% select all the variables in 1 matrix
data = NaN*ones(size(data_lib,1), length(vars));
printvars = {};
for vv = 1:length(vars)
    col = find(strcmp(labels_lib, vars(vv)));
    data(:,vv) = data_lib(:, col);
    printvars = [printvars, printlabels_lib(col)];
end
% timing restrictions
data = data(logical(time >= 1954) & logical(time <= 2019), :);
% target time series, will be called by .mod file
y_obs = data(:,1);
pi_obs = data(:,2);
r_obs = data(:,3)/4;
qe_obs = data(:,4);
save(strcat(folder,'data_for_dynare.mat'),'y_obs','pi_obs','r_obs','qe_obs')

%% Estimation (calling dynare)
dir = cd;
cd(strcat(dir, '/', folder))

dynare nk4

cd(dir)

%% Extract impulse responses and save results
IRF1 = [oo_.irfs.y_epsi', oo_.irfs.pi_epsi', oo_.irfs.r_epsi'];
IRF1(:,3) = IRF1(:,3)*4; %in the model, interest rate is defined per quarter
IRF1 = IRF1 ./ IRF1(1,3);
IRF1bands = repmat(IRF1, 1, 1, 2);
IRF2 = (-10)*[oo_.irfs.y_epsqe', oo_.irfs.pi_epsqe', oo_.irfs.qe_epsqe'];
IRF2bands = repmat(IRF2, 1, 1, 2);
plotirf2(IRF1, IRF1bands, IRF2, IRF2bands, {'Output','Inflation','Shock variable'}, strcat(folder,'lin'), {'Int. rate (p.p.)','QE (% of GDP)'}, 'SouthEast')
save(strcat(folder, 'out.mat'), 'IRF1','IRF2','oo_')

