
%%% This script and the functions used are still work-in-progress %%%
%%% Bug reports welcome                                           %%%

addpath('./_tbx/var_tbx')
addpath('./_tbx/xtratools')

%% Settings
folder = 'c81_counterfactuals/';
p = 12;
h = 48;
c_case = 0;
vars = {'GDPgap','Unemp','CoreCPIGr12','FFR'};
exvars = {};
n = length(vars);
ident = 'chol';
shockpos = 4; % position of the shock in the Cholesky ordering
shocksize = 0.75; % 0 = one standard deviation, all else: absolute values
state.nonlinear = 'no';
nboot = 1000;
alpha = 66; %confidence level


%% Load data, generate lags, subset relevant subsample, etc.
load('data/data_m_ready.mat')
subset_data;


%% Estimate matrices A, Omega, S, dynamic multipliers
VAR = estimateVAR(data, p, c_case, exdata);
VAR.C = dyn_multipliers(VAR, h); % not identified

%% Identification: Cholesky
% organization
VAR.ident = ident;
VAR.shock = zeros(n,1);
VAR.shock(shockpos) = 1;
if shocksize ~= 0  % absolute values, e.g. 25bp = 0.25
    VAR.shock = VAR.shock ./ VAR.S(shockpos, shockpos) .* shocksize;
end
% Cholesky decomposition (already done in estimation function)
disp(VAR.S)
eps = VAR.u*inv(VAR.S);
VAR.eps = eps(:, shockpos);
clear eps
% impulse responses
VAR.IRF = zeros(h,n);
for hh=1:h
    VAR.IRF(hh,:) = (VAR.C(:,:,hh)*VAR.S*VAR.shock)';
end

%% Bootstrap (need all `nboot' number of coeff)
[VAR.IRFbands, Boot] = bootstrapVAR(VAR, nboot, alpha, 'residual');
plotirf1(VAR.IRF, VAR.IRFbands, printvars, strcat(folder,'lin'))


%% Construct counterfactual 1: Expansionary policy shock of -1 in Jan of 2017
cf.T = find(abs(time-(2017-1/12)) == 0) - p; % period just before the counterfactual starts (Dec 2016) 
cf.hz= 36; %Horizon for the counterfactuals
cf.method = 'shockseries'; % shockseries, targetseries
cf.shockseries = [-.5, 0];
cf.shockpos = shockpos;
CFT = cft_VAR(VAR, Boot, alpha, cf);

CFT.time = time(p+1:end);
plotcft(CFT, -48, 36, printvars, strcat(folder,'lin'), [])
save(strcat(folder, 'cft1_out.mat'), 'VAR' ,'CFT')


%% Construct counterfactual 2: Binding zero lower bound instead of negative shadow rates for 2009-15
cf.T = find(time == 2009) - p;
cf.hz= 36; %Horizon for the counterfactuals
cf.method = 'targetseries'; % shockseries, targetseries
cf.targetseries = [-2:(1/10):3];
cf.shockpos = shockpos;
CFT = cft_VAR(VAR, Boot, alpha, cf);

CFT.time = time(p+1:end);
plotcft(CFT, -48, 36, printvars, strcat(folder,'lin'), [])
save(strcat(folder, 'cft2_out.mat'), 'VAR' ,'CFT')

