
%%% This script and the functions used are still work-in-progress %%%
%%% Bug reports welcome                                           %%%

addpath('./_tbx/var_tbx')
addpath('./_tbx/xtratools')

%% Settings
folder = 'c82_misspecification/';
p = 5;
h = 48;
c_case = 1;
vars = {'GDPgap','Unemp','CoreCPIGr4','FFRshadow'};
exvars = {};
n = length(vars);
ident = 'chol';
shockpos = 4; % not actually needed
shocksize = 0; % not actually needed
state.nonlinear = 'no';
nboot = 1000;
alpha = 90; %confidence level

%% Load data, generate lags, subset relevant subsample, etc.
load('data/data_q_ready.mat')
subset_data;


%% Lag selection criteria
t = size(data, 1);
pmax = round(12*((t/100)^(1/4))); %Schwart's rule of thumb 

for p=1:pmax  
  [AIC,BIC,HQC]=InfCriteria(data, p, c_case, exdata);
   AICs(p) = AIC;
   BICs(p) = BIC;
   HQCs(p) = HQC;
end

tab = array2table([(1:pmax)',AICs',BICs',HQCs']); %Display information criteria
tab.Properties.VariableNames = {'Lags','AIC','BIC','HQC'};
disp(tab)

% Display minimum
[~,pref_AIC] = min(AICs);
[~, pref_BIC] = min(BICs);
[~, pref_HQC] = min(HQCs);
tab = array2table([pref_AIC, pref_BIC, pref_HQC]);
tab.Properties.VariableNames = {'AIC', 'BIC', 'HQC'};
disp('Prefered lag order:')
disp(tab)

% Choose the prefered lag order suggested by BIC 
% p = pref_BIC; thinking goes over blindly following lag selection criteria
clear pref_AIC pref_BIC pref_HQC AIC BIC HQC AICs BICs HQCs

    
%% Estimate matrices A, Omega, S
VAR = estimateVAR(data, p, c_case, exdata);


%% Plot the residuals

%Histogram of the residuals
figure
count=0;
for jn = 1:n
    count=count+1;
    subplot(2,2,count)
    histfit(VAR.u(:,jn))
    title(printvars(jn));    
end

%QQ Plots
figure
count=0;
for jn = 1:n
    count=count+1;
    subplot(2,2,count)
    qqplot(VAR.u(:,jn))
    title(printvars(jn));    
end 


%% Test for Autocorrelation
ac = 12; %Set number of autocovariance terms in the test statistic
Portmanteau(VAR.u,ac,p);


%% Test for Multivariate ARCH
March(VAR.u, p, n);


%% Test for Multivariate normality
Multnorm(VAR.u);
