
%%% This script and the functions used are still work-in-progress %%%
%%% Bug reports welcome                                           %%%

addpath('./_tbx/var_tbx')
addpath('./_tbx/xtratools')

%% Settings
folder = 'c83_var2020/';
p = 12;
h = 48;
c_case = 1;
vars = {'GDPgap','CoreCPIGr12','FFR'};
exvars = {};
n = length(vars);
ident = 'chol';
shockpos = 3; % position of the shock in the Cholesky ordering
shocksize = 0; % 0 = one standard deviation, all else: absolute values
state.nonlinear = 'no';
use2020 = 'yes';  % if this is not defined, default is to exclude data after 2020
nboot = 1000;
alpha = 90; %confidence level


%%% 1. Long series without Covid adjustment

%% Load data, generate lags, subset relevant subsample, etc.
load('data/data_m_ready.mat')
subset_data;

%% Estimate matrices A, Omega, S, dynamic multipliers
VAR = estimateVAR(data, p, c_case, exdata);
VAR.C = dyn_multipliers(VAR, h); % not identified

%% Identification: Cholesky
% organization
VAR.ident = ident;
VAR.shock = zeros(n,1);
VAR.shock(shockpos) = 1;
if shocksize ~= 0  % absolute values, e.g. 25bp = 0.25
    VAR.shock = VAR.shock ./ VAR.S(shockpos, shockpos) .* shocksize;
end
% Cholesky decomposition (already done in estimation function)
disp(VAR.S)
eps = VAR.u*inv(VAR.S);
VAR.eps = eps(:, shockpos);
clear eps
% impulse responses
VAR.IRF = zeros(h,n);
for hh=1:h
    VAR.IRF(hh,:) = (VAR.C(:,:,hh)*VAR.S*VAR.shock)';
end

%% Bootstrap
[VAR.IRFbands] = bootstrapVAR(VAR, nboot, alpha, 'residual');

%% Plot impulse responses and save VAR structure
plotirf1(VAR.IRF, VAR.IRFbands, printvars, strcat(folder,'lin'))
save(strcat(folder, 'longvar_out.mat'), 'VAR')


%%% 1. Long series with Covid adjustment

%% Load data, generate lags, subset relevant subsample, etc.
load('data/data_m_ready.mat')
subset_data;

%Set the time period for the Covid shock
Tcovid = find(abs(time-(2020+(3-1)/12)) == 0); % 1 for March 2020

%% Estimate matrices A, Omega, S, dynamic multipliers
VAR = estimateVAR_covid(data, p, c_case, exdata, Tcovid); % maximum likelihood
VAR.C = dyn_multipliers(VAR, h); % not identified

%% Identification: Cholesky
% organization
VAR.ident = ident;
VAR.shock = zeros(n,1);
VAR.shock(shockpos) = 1;
if shocksize ~= 0  % absolute values, e.g. 25bp = 0.25
    VAR.shock = VAR.shock ./ VAR.S(shockpos, shockpos) .* shocksize;
end
% Cholesky decomposition (already done in estimation function)
disp(VAR.S)
eps = VAR.u*inv(VAR.S);
VAR.eps = eps(:, shockpos);
clear eps
% impulse responses
VAR.IRF = zeros(h,n);
for hh=1:h
    VAR.IRF(hh,:) = (VAR.C(:,:,hh)*VAR.S*VAR.shock)';
end

%% Bootstrap
[VAR.IRFbands] = bootstrapVAR(VAR, nboot, alpha, 'residual');

%% Plot impulse responses and save VAR structure
plotirf1(VAR.IRF, VAR.IRFbands, printvars, strcat(folder,'lin'))
save(strcat(folder, 'longvarcovid_out.mat'), 'VAR')
