function [logLH, A, u, Omega, S]=loglhVAR_covid(par,Y,X,p,T,n,MIN,MAX,Tcovid);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This function computes the concentrated log-likelihood of a VAR,
% augmented with a change in volatility at the time of Covid (March 2020)
%
% Last modified: 07/30/2020
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%% hyperparameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ncp=4;      % number of covid hyperparameters
eta=MIN.eta'+(MAX.eta'-MIN.eta')./(1+exp(-par));

invweights = ones(T,1);     % vector of s_t
invweights(Tcovid)=eta(1);
invweights(Tcovid+1)=eta(2);
if T>Tcovid+1;
    invweights(Tcovid+2:T)=1+(eta(3)-1)*eta(4).^[0:T-Tcovid-2];
end
Y = diag(1./invweights)*Y;
X = diag(1./invweights)*X;
    

%% output
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Estimation of the VAR coefficients

A = (X'*X)\(X'*Y);
u = Y-X*A; % reduced-form residuals

% MLE of VAR residual covariance matrix
Omega=u'*u/T; % variance-covariance matrix 
S = chol(Omega)';  % lower-triangular matrix (Cholesky decomposition)

% concentrated log-LH
logLH=-.5*T*log(det(Omega))- n*sum(log(invweights));
logLH=-logLH;       % switch sign because we are minimizing